# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2013 Mariano Ruiz <mrsarm@gmail.com>
#    Enterprise Objects Consulting (<http://www.eoconsulting.com.ar>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################


from osv import osv

class stock_move(osv.osv):
    _inherit = 'stock.move'

    def _get_unique_prodlot_id(self, cr, uid, prod_id):
        """
        Get the id of the unique Lot with stock available >= 0.
        If more than one Lot match the condition, return False.
        """
        prodlot_obj = self.pool.get('stock.production.lot')
        prodlot_ids = prodlot_obj.search(cr,uid,[('product_id', '=', prod_id),('stock_available', '>', 0)], context=None)
        if len(prodlot_ids) == 1:
            return prodlot_ids[0]
        else:
            prodlot_ids = prodlot_obj.search(cr,uid,[('product_id', '=', prod_id),('stock_available', '=', 0)], context=None)
            if len(prodlot_ids) == 1:
                return prodlot_ids[0]
        return False

    def onchange_product_id(self, cr, uid, ids, prod_id, loc_id, loc_dest_id, address_id):
        result = super(stock_move, self).onchange_product_id(cr, uid, ids,
                            prod_id, loc_id, loc_dest_id, address_id)
        if prod_id:
            default_prodlot_id = self.pool.get('product.product').read(
                                cr,uid,prod_id,['default_prodlot_id'])['default_prodlot_id']
            if default_prodlot_id:
                result['value']['prodlot_id'] = default_prodlot_id[0]
            else:
                result['value']['prodlot_id'] = self._get_unique_prodlot_id(cr, uid, prod_id)
        return result

    def create(self, cr, uid, vals, context={}):
        if 'product_id' in vals:
            default_prodlot_id = self.pool.get('product.product').read(
                                cr,uid,vals['product_id'],['default_prodlot_id'])['default_prodlot_id']
            if default_prodlot_id:
                vals['prodlot_id'] = default_prodlot_id[0]
            else:
                vals['prodlot_id'] = self._get_unique_prodlot_id(cr, uid, vals['product_id'])
        new_id = super(stock_move, self).create(cr, uid, vals, context)
        return new_id

stock_move()
